# -*- coding: utf-8 -*-
##############################################################################
#
#    mojo, a Python library for implementing document based databases
#    Copyright (C) 2013-2014 by Javier Sancho Fernandez <jsf at jsancho dot org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import mojo
import MySQLdb

class Connection(mojo.Connection):
    def __init__(self, *args, **kwargs):
        self._db_con = MySQLdb.connect(*args, **kwargs)

    def query(self, sql):
        cur = self._db_con.cursor()
        cur.execute(sql)
        res = cur.fetchall()
        cur.close()
        cur = None
        return res

    def _get_databases(self):
        return [x[0] for x in self.query("SHOW DATABASES")]

    def _get_tables(self, db_name):
        return [x[0] for x in self.query("SHOW TABLES FROM `%s`" % db_name)]

    def _count_rows(self, db_name, table_name):
        return self.query("SELECT COUNT(*) FROM `%s`.`%s`" % (db_name, table_name))[0][0]

    def _get_sql_field(self, db_name, field):
        if type(field) is tuple:
            return "`%s`.`%s`.`%s`" % (db_name, field[0], field[1])
        elif type(field) is dict:
            return "(%s)" % self._get_sql_query(db_name, field)
        else:
            return str(field)
        
    def _get_sql_query(self, db_name, query):
        sql = "SELECT "
        sql += ",".join([self._get_sql_field(db_name, x) for x in query['select']])

        sql += " FROM "
        sql += ",".join(query['from'])

        if query.get('where'):
            sql += " WHERE "
            where = []
            for cond in query['where']:
                where.append("%s %s %s" % (self._get_sql_field(db_name, cond[0]), cond[1], self._get_sql_field(db_name, cond[2])))
            sql += " AND ".join(where)

        return sql

    def _get_cursor(self, db_name, query):
        cur = self._db_con.cursor()
        cur.execute(self._get_sql_query(db_name, query))
        return cur

    def _next(self, cur):
        return cur.fetchone()
