# -*- coding: utf-8 -*-
##############################################################################
#
#    mojo, a Python library for implementing document based databases
#    Copyright (C) 2013-2014 by Javier Sancho Fernandez <jsf at jsancho dot org>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


class Connection(object):
    def __init__(self, *args, **kwargs):
        self._db_con = None

    def __getattr__(self, db_name):
        return Database(self, db_name)

    def __getitem__(self, *args, **kwargs):
        return self.__getattr__(*args, **kwargs)

    def __repr__(self):
        return "Connection(%s)" % self._db_con

    def _get_databases(self):
        return []

    def database_names(self):
        return [unicode(x) for x in self._get_databases()]

    def _get_tables(self, db_name):
        return []

    def collection_names(self, db_name):
        return list(set([unicode(x.split('$')[0]) for x in filter(lambda x: '$' in x, self._get_tables(db_name))]))

    def _count_rows(self, db_name, table_name):
        return 0

    def _count(self, db_name, table_name):
        try:
            return self._count_rows(db_name, table_name + '$_id')
        except:
            return 0

    def _get_cursor(self, db_name, query):
        # {'select': [('t1$_id', 'id'), {'select': [('t1$c1', 'value')], 'from': ['t1$c1'], 'where': [(('t1$c1', 'id'), '=', ('t1$_id', 'id'))]}], 'from': ['t1$_id']}
        return None

    def _next(self, cursor):
        return None


class Database(object):
    def __init__(self, connection, db_name):
        self.connection = connection
        self.db_name = unicode(db_name)

    def __getattr__(self, table_name):
        return Collection(self, table_name)

    def __getitem__(self, *args, **kwargs):
        return self.__getattr__(*args, **kwargs)

    def __repr__(self):
        return "Database(%r, %r)" % (self.connection, self.db_name)

    def collection_names(self):
        return self.connection.collection_names(self.db_name)


class Collection(object):
    def __init__(self, database, table_name):
        self.database = database
        self.table_name = unicode(table_name)

    def __repr__(self):
        return "Collection(%r, %r)" % (self.database, self.table_name)

    def _get_fields(self):
        tables = self.database.connection._get_tables(self.database.db_name)
        return [unicode(x[x.find('$')+1:]) for x in filter(lambda x: x.startswith('%s$' % self.table_name), tables)]

    def count(self):
        return self.database.connection._count(self.database.db_name, self.table_name)

    def find(self, *args, **kwargs):
        return Cursor(self, *args, **kwargs)


class Cursor(object):
    def __init__(self, collection, spec=None, fields=None, **kwargs):
        self.collection = collection
        self.spec = spec
        self.fields = self._get_fields(fields)
        self.cursor = self._get_cursor()

    def __iter__(self):
        return self

    def _get_fields(self, fields):
        set_all_fields = set(self.collection._get_fields())
        if fields is None:
            res_fields = list(set_all_fields)
        elif type(fields) is dict:
            fields_without_id = filter(lambda x: x[0] != '_id', fields.iteritems())
            if fields_without_id[0][1]:
                first = True
                res_fields = set()
            else:
                first = False
                res_fields = set(set_all_fields)
            for f in fields_without_id:
                if f[1] and f[0] in set_all_fields:
                    if first:
                        res_fields.add(f[0])
                    else:
                        raise Exception(Errors['mix_fields_error'])
                elif not f[1]:
                    if not first:
                        res_fields.discard(f[0])
                    else:
                        raise Exception(Errors['mix_fields_error'])
            if '_id' in fields and not fields['_id']:
                res_fields.discard('_id')
            else:
                res_fields.add('_id')
            res_fields = list(res_fields)
        else:
            set_fields = set(list(fields))
            set_fields.add('_id')
            res_fields = list(set_all_fields.intersection(set_fields))

        return res_fields

    def _get_cursor(self):
        query = {}
        table_id = '%s$_id' % self.collection.table_name

        query['select'] = [(table_id, 'id')]
        for f in filter(lambda x: x != '_id', self.fields):
            table_f = '%s$%s' % (self.collection.table_name, f)
            q = {}
            q['select'] = [(table_f, 'value')]
            q['from'] = [table_f]
            q['where'] = [((table_f, 'id'), '=', (table_id, 'id'))]
            query['select'].append(q)

        query['from'] = [table_id]

        return self.collection.database.connection._get_cursor(self.collection.database.db_name, query)

    def next(self):
        if self.cursor:
            res = self.collection.database.connection._next(self.cursor)
            if res is None:
                raise StopIteration
            else:
                document = {}
                if '_id' in self.fields:
                    document['_id'] = res[0]
                fields_without_id = filter(lambda x: x != '_id', self.fields)
                for i in xrange(len(fields_without_id)):
                    if not res[i + 1] is None:
                        document[fields_without_id[i]] = res[i + 1]
                return document
        else:
            return None


Errors = {
    'mix_fields_error': 'You cannot currently mix including and excluding fields. Contact us if this is an issue.',
    }
